@@ -2,7 +2,7 @@ package RedisDB;
 
 use warnings;
 use strict;
-our $VERSION = "0.31";
+our $VERSION = "1.00";
 $VERSION = eval $VERSION;
 
 use RedisDB::Error;
@@ -13,6 +13,7 @@ use POSIX qw(:errno_h);
 use Config;
 use Carp;
 use Try::Tiny;
+use Encode qw();
 
 =head1 NAME
 
@@ -69,6 +70,12 @@ number of seconds, module will croak. Note, that some OSes do not support
 SO_RCVTIMEO, and SO_SNDTIMEO socket options, in this case timeout will not
 work.
 
+=item utf8
+
+Assume that all data on the server encoded in UTF-8. As result all strings will
+be converted to UTF-8 before sending to server, and all results will be decoded
+from UTF-8. See L</"UTF-8 SUPPORT">.
+
 =item lazy
 
 by default I<new> establishes connection to the server. If this parameter is
@@ -116,7 +123,7 @@ replies.
 
 sub execute {
     my $self = shift;
-    croak "You can't use RedisDB::execute while in pipelining mode."
+    croak "You can't use RedisDB::execute when you have replies to fetch."
       if $self->replies_to_fetch;
     croak "This function is not available in subscription mode." if $self->{_subscription_loop};
     my $cmd = uc shift;
@@ -130,7 +137,7 @@ sub _connect {
     my $self = shift;
 
     # this is to prevent recursion
-    croak "Couldn't connect to the redis-server."
+    confess "Couldn't connect to the redis-server."
       . " Connection was immediately closed by the server."
       if $self->{_in_connect}++;
 
@@ -152,10 +159,7 @@ sub _connect {
     }
 
     if ( $self->{timeout} ) {
-        my $timeout =
-          $Config{longsize} == 4
-          ? pack( 'LL', $self->{timeout}, 0 )
-          : pack( 'QQ', $self->{timeout} );
+        my $timeout = pack( 'L!L!', $self->{timeout}, 0 );
         try {
             defined $self->{_socket}->sockopt( SO_RCVTIMEO, $timeout )
               or die "Can't set timeout: $!";
@@ -169,6 +173,7 @@ sub _connect {
 
     $self->{_callbacks}         = [];
     $self->{_subscription_loop} = 0;
+    delete $self->{_server_version};
     if ( $self->{database} ) {
         $self->send_command( "SELECT", $self->{database}, IGNORE_REPLY() );
     }
@@ -203,7 +208,7 @@ sub _recv_data_nb {
             next if $! == EINTR;
 
             # die on any other error
-            die "Error reading from server: $!";
+            confess "Error reading from server: $!";
         }
         elsif ( $buf ne '' ) {
 
@@ -216,8 +221,8 @@ sub _recv_data_nb {
             # server closed connection. Check if some data was lost.
             1 while $self->{_buffer} and $self->_parse_reply;
 
-            # if there's some replies lost
-            die "Server closed connection. Some data was lost."
+            # if there are some replies lost
+            confess "Server closed connection. Some data was lost."
               if @{ $self->{_callbacks} }
                   or $self->{_in_multi};
 
@@ -286,7 +291,7 @@ sub send_command {
         };
     }
 
-    my $request = _build_redis_request( $command, @_ );
+    my $request = _build_redis_request( $self, $command, @_ );
     $self->_connect unless $self->{_socket} and $self->{_pid} == $$;
 
     # Here we reading received data and storing it in the _buffer,
@@ -294,7 +299,7 @@ sub send_command {
     # and reconnect if not
     $self->_recv_data_nb;
 
-    defined $self->{_socket}->send($request) or die "Can't send request to server: $!";
+    defined $self->{_socket}->send($request) or confess "Can't send request to server: $!";
     push @{ $self->{_callbacks} }, $callback;
     return 1;
 }
@@ -371,11 +376,11 @@ sub mainloop {
     my $self = shift;
 
     while ( @{ $self->{_callbacks} } ) {
-        die "You can't call mainloop in the child process" unless $self->{_pid} == $$;
+        croak "You can't call mainloop in the child process" unless $self->{_pid} == $$;
         my $ret = $self->{_socket}->recv( my $buffer, 4096 );
         unless ( defined $ret ) {
             next if $! == EINTR;
-            croak "Error reading reply from server: $!";
+            confess "Error reading reply from server: $!";
         }
         if ( $buffer ne '' ) {
 
@@ -386,7 +391,7 @@ sub mainloop {
         else {
 
             # disconnected
-            die "Server unexpectedly closed connection before sending full reply";
+            confess "Server unexpectedly closed connection before sending full reply";
         }
     }
     return;
@@ -402,15 +407,15 @@ sub get_reply {
     my $self = shift;
 
     while ( not @{ $self->{_replies} } ) {
-        die "We are not waiting for reply"
+        croak "We are not waiting for reply"
           unless $self->{_to_be_fetched}
               or $self->{_subscription_loop};
-        die "You can't read reply in child process" unless $self->{_pid} == $$;
+        croak "You can't read reply in child process" unless $self->{_pid} == $$;
         while ( not $self->_parse_reply ) {
             my $ret = $self->{_socket}->recv( my $buffer, 4096 );
             unless ( defined $ret ) {
                 next if $! == EINTR;
-                croak "Error reading reply from server: $!";
+                confess "Error reading reply from server: $!";
             }
             if ( $buffer ne '' ) {
 
@@ -420,7 +425,7 @@ sub get_reply {
             else {
 
                 # disconnected
-                die "Server unexpectedly closed connection before sending full reply";
+                confess "Server unexpectedly closed connection before sending full reply";
             }
         }
     }
@@ -467,6 +472,23 @@ sub selected_database {
     shift->{database};
 }
 
+=head2 $self->reset_connection
+
+Resets connection. This closes existing connection and drops all previously
+sent requests. After invoking this method the object returns to the same state
+as it was returned by the constructor.
+
+=cut
+
+sub reset_connection {
+    my $self = shift;
+    delete $self->{$_} for grep /^_/, keys %$self;
+    $self->{_replies}       = [];
+    $self->{_callbacks}     = [];
+    $self->{_to_be_fetched} = 0;
+    return;
+}
+
 =head2 $self->version
 
 Return the version of the server the client is connected to. The version is
@@ -479,7 +501,7 @@ sub version {
     my $self = shift;
     my $info = $self->info;
     $info->{redis_version} =~ /^([0-9]+)[.]([0-9]+)(?:[.]([0-9]+))?/
-      or die "Can't parse version string: $info->{redis_version}";
+      or croak "Can't parse version string: $info->{redis_version}";
     $self->{_server_version} = $1 + 0.001 * $2 + ( $3 ? 0.000001 * $3 : 0 );
     return $self->{_server_version};
 }
@@ -593,6 +615,39 @@ sub shutdown {
     return;
 }
 
+=head1 UTF-8 SUPPORT
+
+The redis protocol is designed to work with the binary data, both keys and
+values are encoded in the same way as sequences of octets.  By default this
+module expects all data to be just strings of bytes. There is an option to
+treat all data as UTF-8 strings. If you pass I<utf8> parameter to the
+constructor, module will encode all strings to UTF-8 before sending them to
+server, and will decode all strings received from server from UTF-8. This has
+following repercussions you should be aware off: first, you can't store binary
+data on server with this option on, it would be treated as a sequence of latin1
+characters, and would be converted into a corresponding sequence of UTF-8 encoded
+characters; second, if data returned by the server is not a valid UTF-8 encoded
+string, the module will croak, and you will have to reinitialize the
+connection. Generally, I would recommend to write a wrapper around L<RedisDB> instead of
+setting I<utf8> option.
+
+=cut
+
+=head1 ERROR HANDLING
+
+If an error happens which the module can't handle, it will croak. It may
+happen as a result of a network error, or invalid data encoding, or if the
+server returned an error reply. In some cases the RedisDB object after throwing
+an exception will be left in inconsistant state. If you want to continue using
+the object after getting an exception, you should invoke the
+L</reset_connection> method on it. This will drop current connection and all
+outstanding requests, so the object will return to the same state it was just
+after creation with the L</new> method. If the connection was in subscription
+mode, you will have to restore all the subscriptions, if it was in the middle
+of transaction, you will have to start the transaction again.
+
+=cut
+
 =head1 HANDLING OF SERVER DISCONNECTS
 
 Redis server may close a connection if it was idle for some time, also the
@@ -630,9 +685,9 @@ I<get_reply>, I<get_all_replies>.  Calling wrapper method, like
 C<< $redis->get('key') >>, will also trigger check as internally wrapper methods
 use methods listed above.
 
-If you invoke I<send_command> without a callback argument, you have to fetch
-reply later explicitly using I<get_reply> method. This is how synchronous
-I<execute> is implemented, basically it is:
+Also you can omit callback argument when invoke I<send_command>. In this case
+you have to fetch reply later explicitly using I<get_reply> method. This is how
+synchronous I<execute> is implemented, basically it is:
 
     sub execute {
         my $self = shift;
@@ -640,8 +695,9 @@ I<execute> is implemented, basically it is:
         return $self->get_reply;
     }
 
-That is why it is not allowed to call I<execute> if you have not got replies to
-all commands sent previously with I<send_command> without callback.
+That is why it is not allowed to call I<execute> unless you have got replies to
+all commands sent previously with I<send_command> without callback. Using
+I<send_command> without callback is not recommended.
 
 Sometimes you are not interested in replies sent by the server, e.g. SET
 command usually just return 'OK', in this case you can pass to I<send_command>
@@ -784,7 +840,7 @@ sub subscription_loop {
       unless ( keys %{ $self->{_subscribed} } or keys %{ $self->{_psubscribed} } );
 
     while ( my $msg = $self->get_reply ) {
-        die "Expected multi-bulk reply, but got $msg" unless ref $msg;
+        confess "Expected multi-bulk reply, but got $msg" unless ref $msg;
         if ( $msg->[0] eq 'message' ) {
             $self->{_subscribed}{ $msg->[1] }( $self, $msg->[1], undef, $msg->[2] );
         }
@@ -806,7 +862,7 @@ sub subscription_loop {
             last unless $msg->[2] or %{ $self->{_subscribed} };
         }
         else {
-            die "Got unknown reply $msg->[0] in subscription mode";
+            confess "Got unknown reply $msg->[0] in subscription mode";
         }
     }
     $self->{_to_be_fetched} = 0;
@@ -962,13 +1018,17 @@ sub discard {
 #
 # Builds unified redis request from given I<$command> and I<@arguments>.
 sub _build_redis_request {
+    my $self  = shift;
     my $nargs = @_;
 
-    use bytes;
     my $req = "*$nargs\015\012";
-    while ( $nargs-- ) {
-        my $arg = shift;
-        $req .= '$' . length($arg) . "\015\012" . $arg . "\015\012";
+    if ( $self->{utf8} ) {
+        $req .= '$' . length($_) . "\015\012" . $_ . "\015\012"
+          for map { Encode::encode( 'UTF-8', $_, Encode::FB_CROAK | Encode::LEAVE_SRC ) } @_;
+    }
+    else {
+        use bytes;
+        $req .= '$' . length($_) . "\015\012" . $_ . "\015\012" for @_;
     }
     return $req;
 }
@@ -1059,6 +1119,14 @@ sub _parse_reply {
             return unless length $self->{_buffer} >= 2 + $self->{_parse_bulk_len};
             my $bulk = substr( $self->{_buffer}, 0, $self->{_parse_bulk_len}, '' );
             substr $self->{_buffer}, 0, 2, '';
+            if ( $self->{utf8} ) {
+                try {
+                    $bulk = Encode::decode( 'UTF-8', $bulk, Encode::FB_CROAK | Encode::LEAVE_SRC );
+                }
+                catch {
+                    croak "Couldn't decode reply from the server, invalid UTF-8: '$bulk'";
+                };
+            }
             if ( $self->{_parse_reply}[0] eq '$' ) {
                 $self->{_parse_reply}[1] = $bulk;
                 return $self->_reply_completed;
@@ -1118,8 +1186,7 @@ sub _parse_reply {
                 $self->{_parse_reply} = ['*'];
             }
             else {
-                die "Invalid multi-bulk reply. Expected '\$' or ':' but got $char"
-                  ;    # $self->{_buffer}";
+                die "Invalid multi-bulk reply. Expected '\$' or ':' but got $char";
             }
             $repeat = 1;
         }
@@ -1222,7 +1289,7 @@ Pavel Shaydo, C<< <zwon at cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2011 Pavel Shaydo.
+Copyright 2011, 2012 Pavel Shaydo.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
